#ifndef POLY2D1_HPP
#define POLY2D1_HPP

#include <cmath>
#include <iostream>

#include "complexe.hpp"

template <class K> class POLY2D1 {
    private :
        K a;
        K b1;
        K b2;
        K c;
        K d1;
        K d2;
    public:
        POLY2D1();
        POLY2D1(K, K, K, K, K ,K);
        POLY2D1(const POLY2D1 & p);
        ~POLY2D1();
        K geta();
        K getb1();
        K getb2();
        K getc();
        K getd1();
        K getd2();
        void seta(K);
        void setb1(K);
        void setb2(K);
        void setc(K);
        void setd1(K);
        void setd2(K);
        K operator ()(K,K);
        POLY2D1 operator +(POLY2D1 z);
        POLY2D1 operator *(POLY2D1 z);
        POLY2D1 & operator =(const POLY2D1 & z);
};


template<class K> POLY2D1<K>::POLY2D1() {
    a = 0.0;
    b1 = 0.0; 
    b2 = 0.0;
    c = 0.0; 
    d1 = 0.0; 
    d2 = 0.0;
}

template<class K> POLY2D1<K>::POLY2D1(K w, K x, K y, K z, K z1, K z2) {
    a = w;
    b1 = x; 
    b2 = y;
    c = z; 
    d1 = z1;
    d2 = z2;
}

template<class K> POLY2D1<K>::POLY2D1(const POLY2D1<K> & p) {
    a = p.a;
    b1 = p.b1; 
    b2 = p.b2;
    c = p.c; 
    d1 = p.d1;
    d2 = p.d2;
}

template<class K> POLY2D1<K> & POLY2D1<K>:: operator =(const POLY2D1<K> & p) {
    if ( this != &p){
        a = p.a;
        b1 = p.b1; 
        b2 = p.b2;
        c = p.c; 
        d1 = p.d1;
        d2 = p.d2;
    }
    return *this;
}

template<class K> POLY2D1<K>::~POLY2D1() { 
}

template<class K> K POLY2D1<K>::geta() {
    return a;
}
template<class K> K POLY2D1<K>::getb1() {
    return b1;
}
template<class K> K POLY2D1<K>::getb2() {
    return b2;
}
template<class K> K POLY2D1<K>::getc() {
    return c;
}
template<class K> K POLY2D1<K>::getd1() {
    return d1;
}
template<class K> K POLY2D1<K>::getd2() {
    return d2;
}
template<class K> void POLY2D1<K>::seta(K x) {
    a=x;
}
template<class K> void POLY2D1<K>::setb1(K x) {
    b1=x;
}
template<class K> void POLY2D1<K>::setb2(K x) {
    b2=x;
}
template<class K> void POLY2D1<K>::setc(K x) {
    c=x;
}
template<class K> void POLY2D1<K>::setd1(K x) {
    d1=x;
}
template<class K> void POLY2D1<K>::setd2(K x) {
    d2=x;
}

template<class K> K POLY2D1<K>::operator()(K x,K y){
    K res;
    res = a+b1*x+b2*y+c*x*y+d1*x*x+d2*y*y;
    return res;
}

template<class K> POLY2D1<K> POLY2D1<K>::operator+(POLY2D1<K> p){
    POLY2D1<K> res;
    res.a=a+p.a;
    res.b1=b1+p.b1;
    res.b2=b2+p.b2;
    res.c=c+p.c;
    res.d1=d1+p.d1;
    res.d2=d2+p.d2;
    return res;
}

template<> 
POLY2D1<double> POLY2D1<double>::operator*(POLY2D1<double> p){
    POLY2D1<double> res;
    double eps  = 0.00001;
    if( abs(d1)> eps || abs(d2)> eps || abs(c)> eps){
        if( abs(p.d1)> eps || abs(p.d2)> eps || abs(p.c)> eps || abs(p.b1)> eps || abs(p.b2)> eps){
            cout<<"product non possible"<<endl;exit(0);
        } else {
            res.a = a*p.a;
            res.b1 = b1*p.a;
            res.b2 = b2*p.a;
            res.c = c*p.a;
            res.d1 = d1*p.a;
            res.d2 = d2*p.a;
        }
    } else if( abs(p.d1)> eps || abs(p.d2)> eps || abs(p.c)> eps){
        if( abs(d1)> eps || abs(d2)> eps || abs(c)> eps || abs(b1)> eps || abs(b2)> eps){
            cout<<"product non possible"<<endl;exit(0);
        } else {
            res.a = p.a*a;
            res.b1 = p.b1*a;
            res.b2 = p.b2*a;
            res.c = p.c*a;
            res.d1 = p.d1*a;
            res.d2 = p.d2*a;
        }
    } else {
        res.a = a*p.a;
        res.b1 = a*p.b1+p.a*b1;
        res.b2 = a*p.b2+p.a*b2;
        res.c = b1*p.b2+p.b1*b2;
        res.d1 = b1*p.b1;
        res.d2 = b2*p.b2;
    }
    return res;
}


template <class K> class POLY2D2 {
    private :
        K coefs[6];
    public:
        POLY2D2();
        POLY2D2(K, K, K, K, K ,K);
        POLY2D2(const POLY2D2 & p);
        ~POLY2D2();
        K get(int);
        void set(K,int);
        K operator ()(K,K);
        POLY2D2 operator +(POLY2D2 z);
        POLY2D2 operator *(POLY2D2 z);
        POLY2D2 & operator =(const POLY2D2 & z);
};

template<class K> POLY2D2<K>::POLY2D2() {
    for(int i=0;i<6;i++){
        coefs[i]=K();
    }
}

template<class K> POLY2D2<K>::POLY2D2(K w, K x, K y, K z, K z1, K z2) {
    coefs[0] = w;
    coefs[1] = x; 
    coefs[2] = y;
    coefs[3] = z; 
    coefs[4] = z1;
    coefs[5] = z2;
}

template<class K> POLY2D2<K>::POLY2D2(const POLY2D2<K> & p) {
    for(int i=0;i<6;i++){
        coefs[i]=p.coefs[i];
    }
}

template<class K> POLY2D2<K> & POLY2D2<K>:: operator =(const POLY2D2<K> & p) {
    if ( this != &p){
        for(int i=0;i<6;i++){
            coefs[i]=p.coefs[i];
        }
    }
    return *this;
}

template<class K> POLY2D2<K>::~POLY2D2() { 
}

template<class K> K POLY2D2<K>::get(int i) {
    return coefs[i];
}

template<class K> void POLY2D2<K>::set(K x,int i) {
    coefs[i]=x;
}


template<class K> K POLY2D2<K>::operator()(K x,K y){
    K res = coefs[0];
    res= coefs[1]*x+coefs[2]*y+coefs[3]*x*y+coefs[4]*x*x+coefs[5]*y*y;
    return res;
}

template<class K> POLY2D2<K> POLY2D2<K>::operator+(POLY2D2<K> p){
    POLY2D2<K> res;
    for(int i=0;i<6;i++){
        res.coefs[i]=p.coefs[i]+coefs[i];
    }
    return res;
}

template<> 
POLY2D2<double> POLY2D2<double>::operator*(POLY2D2<double> p){
    POLY2D2<double> res;
    double eps  = 0.00001;
    if( abs(coefs[4])> eps || abs(coefs[5])> eps || abs(coefs[3])> eps){
        if( abs(p.coefs[4])> eps || abs(p.coefs[5])> eps || abs(p.coefs[3])> eps || abs(p.coefs[1])> eps || abs(p.coefs[2])> eps){
            cout<<"product non possible"<<endl;exit(0);
        } else {
            for(int i=0;i<6;i++){
                res.coefs[i]=coefs[i]*p.coefs[0];
            }
        }
    } else if( abs(p.coefs[4])> eps || abs(p.coefs[5])> eps || abs(p.coefs[3])> eps){
        if( abs(coefs[4])> eps || abs(coefs[5])> eps || abs(coefs[3])> eps || abs(coefs[1])> eps || abs(coefs[2])> eps){
            cout<<"product non possible"<<endl;exit(0);
        } else {
            for(int i=0;i<6;i++){
                res.coefs[i]=p.coefs[i]*coefs[0];
            }
        }
    } else {
        res.coefs[0] = coefs[0]*p.coefs[0];
        res.coefs[1] = coefs[0]*p.coefs[1]+p.coefs[0]*coefs[1];
        res.coefs[2] = coefs[0]*p.coefs[2]+p.coefs[0]*coefs[2];
        res.coefs[3] = coefs[1]*p.coefs[2]+p.coefs[1]*coefs[2];
        res.coefs[1] = coefs[1]*p.coefs[1];
        res.coefs[2] = coefs[2]*p.coefs[2];
    }
    return res;
}
#endif
