#include <cmath>
#include <iostream>
using namespace std;

#include "complexe.hpp"
#include "racine_unite.hpp"

#ifndef M_PI
    #define M_PI 3.14159265358979323846
#endif

racineUnite::racineUnite() {
    m_k = 0;
    m_n = 1; 
}

racineUnite::racineUnite(int k, int n) {
    if (n != 0) {
        m_k = k % n;
        m_n = n;
    } else {
        cout << "Erreur : initialisation par défaut effectuée" << endl;
        m_k = 0;
        m_n = 1;
    }
}

racineUnite::racineUnite(const racineUnite & z) {
    m_k = z.m_k;
    m_n = z.m_n;
}

racineUnite::~racineUnite() {
}

int racineUnite::getk(){
    return m_k;
}
int racineUnite::getn(){
    return m_n;
}

void racineUnite::setk(int k){
    m_k = k;
}
void racineUnite::setn(int n){
    m_n = n;
}
       
// Calcul du nombre complexe associé
complexe racineUnite::calculeComplexe() {
    complexe res = complexe(cos((2. * m_k * M_PI) / (double)m_n), sin((2. * m_k * M_PI) / (double)m_n));
    return res;
}

racineUnite racineUnite::operator *(racineUnite z) {
    racineUnite res;
    res.m_k = m_k * z.m_n + z.m_k * m_n;
    res.m_n = m_n * z.m_n;
    return res;
}

racineUnite & racineUnite::operator =(const racineUnite & z) {
    if (&z != this) {
        m_k = z.m_k;
        m_n = z.m_n;
    }
    return *this;
}

ostream & operator <<(ostream & out, const racineUnite & z) {
    out << "exp(2i * pi * " << z.m_k << " / " << z.m_n << ")" << endl;
    return out;
}


